@@ -6,12 +6,21 @@
  * Version:	@(#)dowall.c  2.85-5  02-Jul-2003  miquels@cistron.nl
  *
  *		This file is part of the sysvinit suite,
- *		Copyright 1991-2003 Miquel van Smoorenburg.
+ *		Copyright (C) 1991-2003 Miquel van Smoorenburg.
  *
- *		This program is free software; you can redistribute it and/or
- *		modify it under the terms of the GNU General Public License
- *		as published by the Free Software Foundation; either version
- *		2 of the License, or (at your option) any later version.
+ *		This program is free software; you can redistribute it and/or modify
+ *		it under the terms of the GNU General Public License as published by
+ *		the Free Software Foundation; either version 2 of the License, or
+ *		(at your option) any later version.
+ *
+ *		This program is distributed in the hope that it will be useful,
+ *		but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *		GNU General Public License for more details.
+ *
+ *		You should have received a copy of the GNU General Public License
+ *		along with this program; if not, write to the Free Software
+ *		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -27,6 +36,14 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <setjmp.h>
+#include <paths.h>
+
+#ifndef _PATH_DEV
+# define _PATH_DEV	"/dev/"
+#endif
+#ifndef HOST_NAME_MAX
+# define HOST_NAME_MAX	255
+#endif
 
 static sigjmp_buf jbuf;
 
@@ -34,7 +51,11 @@ static sigjmp_buf jbuf;
  *	Alarm handler
  */
 /*ARGSUSED*/
+# ifdef __GNUC__
+static void handler(int arg __attribute__((unused)))
+# else
 static void handler(int arg)
+# endif
 {
 	siglongjmp(jbuf, 1);
 }
@@ -43,7 +64,7 @@ static void handler(int arg)
 /*
  *	Print a text, escape all characters not in Latin-1.
  */
-static void feputs(char *line, FILE *fp)
+static void feputs(const char *line, FILE *fp)
 {
 	unsigned char		*p;
 
@@ -65,7 +86,7 @@ static void getuidtty(char **userp, char **ttyp)
 	uid_t			uid;
 	char			*tty;
 	static char		uidbuf[32];
-	static char		ttynm[32];
+	static char		ttynm[UT_LINESIZE + 4];
 	static int		init = 0;
 
 	if (!init) {
@@ -75,13 +96,25 @@ static void getuidtty(char **userp, char **ttyp)
 			uidbuf[0] = 0;
 			strncat(uidbuf, pwd->pw_name, sizeof(uidbuf) - 1);
 		} else {
+                        /* Using variable number of data parameters in one
+                        function makes the Clang compiler cry. -- Jesse
 			sprintf(uidbuf, uid ? "uid %d" : "root", (int)uid);
+                        */
+                        if (uid)
+                           sprintf(uidbuf, "uid %d", (int) uid);
+                        else
+                           sprintf(uidbuf, "root");
 		}
 
 		if ((tty = ttyname(0)) != NULL) {
-			if (strncmp(tty, "/dev/", 5) == 0)
-				tty += 5;
-			sprintf(ttynm, "(%.28s) ", tty);	
+			const size_t plen = strlen(_PATH_DEV);
+			if (strncmp(tty, _PATH_DEV, plen) == 0) {
+				tty += plen;
+				if (tty[0] == '/')
+					tty++;
+			}
+			snprintf(ttynm, sizeof(ttynm), "(%.*s) ",
+				 UT_LINESIZE, tty);
 		} else
 			ttynm[0] = 0;
 		init++;
@@ -123,14 +156,15 @@ static int file_isatty(const char *fname)
 /*
  *	Wall function.
  */
-void wall(char *text, int fromshutdown, int remote)
+void wall(const char *text, int remote)
 {
 	FILE			*tp;
 	struct sigaction	sa;
 	struct utmp		*utmp;
 	time_t			t;
-	char			term[UT_LINESIZE+6];
-	char			line[81];
+	char			term[UT_LINESIZE+ strlen(_PATH_DEV) + 1];
+	char			line[256];
+	char			hostname[HOST_NAME_MAX+1];
 	char			*date, *p;
 	char			*user, *tty;
 	int			fd, flags;
@@ -144,21 +178,33 @@ void wall(char *text, int fromshutdown, int remote)
 
 	getuidtty(&user, &tty);
 
+	/* Get and report current hostname, to make it easier to find
+	   out which machine is being shut down. */
+	if (0 != gethostname(hostname, sizeof(hostname))) {
+		strncpy(hostname, "[unknown]", sizeof(hostname)-1);
+	}
+	/* If hostname is truncated, it is unspecified if the string
+	   is null terminated or not.  Make sure we know it is null
+	   terminated. */
+	hostname[sizeof(hostname)-1] = 0;
+
 	/* Get the time */
 	time(&t);
 	date = ctime(&t);
+        if (! date)
+           date = " ";
 	for(p = date; *p && *p != '\n'; p++)
 		;
 	*p = 0;
 	
 	if (remote) {
 		snprintf(line, sizeof(line),
-			"\007\r\nRemote broadcast message (%s):\r\n\r\n",
+			"\r\nRemote broadcast message (%s):\r\n\r\n",
 			date);
 	} else {
 		snprintf(line, sizeof(line),
-			"\007\r\nBroadcast message from %s %s(%s):\r\n\r\n",
-			user, tty, date);
+			"\r\nBroadcast message from %s@%s %s(%s):\r\n\r\n",
+			user, hostname, tty, date);
 	}
 
 	/*
@@ -178,11 +224,11 @@ void wall(char *text, int fromshutdown, int remote)
 	while ((utmp = getutent()) != NULL) {
 		if(utmp->ut_type != USER_PROCESS ||
 		   utmp->ut_user[0] == 0) continue;
-		if (strncmp(utmp->ut_line, "/dev/", 5) == 0) {
+		if (strncmp(utmp->ut_line, _PATH_DEV, strlen(_PATH_DEV)) == 0) {
 			term[0] = 0;
-			strncat(term, utmp->ut_line, UT_LINESIZE);
+			strncat(term, utmp->ut_line, sizeof(term)-1);
 		} else
-			snprintf(term, sizeof(term), "/dev/%.*s",
+			snprintf(term, sizeof(term), _PATH_DEV "%.*s",
 				UT_LINESIZE, utmp->ut_line);
 		if (strstr(term, "/../")) continue;
 
